@@ -2,14 +2,22 @@ import ast
 import atexit
 import configparser
 import contextlib
+import errno
+import locale
 import os
 import shutil
+import stat
+import subprocess as sp
 import sys
+import time
+import warnings
 from collections.abc import Iterable, Mapping
 from contextlib import ExitStack
 from functools import lru_cache
+from itertools import count
 from os import scandir
 from pathlib import Path
+from typing import Callable, Optional
 from urllib.parse import parse_qs, urlparse, urlunparse
 from weakref import finalize
 
@@ -17,6 +25,7 @@ import attr
 from distlib.wheel import Wheel
 from pep517 import envbuild, wrappers
 from pip._internal.network.download import Downloader
+from pip._internal.operations.prepare import unpack_url
 from pip._internal.utils.temp_dir import global_tempdir_manager
 from pip._internal.utils.urls import url_to_path
 from pip._vendor.packaging.markers import Marker
@@ -29,14 +38,12 @@ from pip._vendor.pkg_resources import (
     find_distributions,
 )
 from platformdirs import user_cache_dir
-from vistir.contextmanagers import cd, temp_path
-from vistir.misc import run
-from vistir.path import create_tracked_tempdir, rmtree
 
 from ..environment import MYPY_RUNNING
 from ..exceptions import RequirementError
+from ..fileutils import cd, create_tracked_tempdir, temp_path
 from ..utils import get_pip_command
-from .old_pip_utils import old_unpack_url
+from .old_pip_utils import _copy_source_tree
 from .utils import (
     get_default_pyproject_backend,
     get_name_variants,
@@ -53,7 +60,6 @@ if MYPY_RUNNING:
         Dict,
         Generator,
         List,
-        Optional,
         Sequence,
         Set,
         Text,
@@ -67,11 +73,7 @@ if MYPY_RUNNING:
     from pip._vendor.packaging.requirements import Requirement as PackagingRequirement
     from pip._vendor.pkg_resources import DistInfoDistribution, EggInfoDistribution
     from pip._vendor.requests import Session
-
-    try:
-        from setuptools.dist import Distribution
-    except ImportError:
-        from distutils.core import Distribution
+    from setuptools.dist import Distribution
 
     TRequirement = TypeVar("TRequirement")
     RequirementType = TypeVar(
@@ -98,16 +100,7 @@ def pep517_subprocess_runner(cmd, cwd=None, extra_environ=None):
     if extra_environ:
         env.update(extra_environ)
 
-    run(
-        cmd,
-        cwd=cwd,
-        env=env,
-        block=True,
-        combine_stderr=True,
-        return_object=False,
-        write_to_stdout=False,
-        nospin=True,
-    )
+    sp.run(cmd, cwd=cwd, env=env, stdout=sp.PIPE, stderr=sp.STDOUT)
 
 
 class BuildEnv(envbuild.BuildEnvironment):
@@ -121,14 +114,8 @@ class BuildEnv(envbuild.BuildEnvironment):
             "--prefix",
             self.path,
         ] + list(reqs)
-        run(
-            cmd,
-            block=True,
-            combine_stderr=True,
-            return_object=False,
-            write_to_stdout=False,
-            nospin=True,
-        )
+
+        sp.run(cmd, stderr=sp.PIPE, stdout=sp.PIPE)
 
 
 class HookCaller(wrappers.Pep517HookCaller):
@@ -144,6 +131,229 @@ class HookCaller(wrappers.Pep517HookCaller):
         self.backend_path = backend_path
 
 
+def get_value_from_tuple(value, value_type):
+    try:
+        import winreg
+    except ImportError:
+        import _winreg as winreg
+    if value_type in (winreg.REG_SZ, winreg.REG_EXPAND_SZ):
+        if "\0" in value:
+            return value[: value.index("\0")]
+        return value
+    return None
+
+
+def is_readonly_path(fn: os.PathLike) -> bool:
+    """check if a provided path exists and is readonly.
+
+    permissions check is `bool(path.stat & stat.s_iread)` or `not
+    os.access(path, os.w_ok)`
+    """
+    if os.path.exists(fn):
+        file_stat = os.stat(fn).st_mode
+        return not bool(file_stat & stat.s_iwrite) or not os.access(fn, os.w_ok)
+    return False
+
+
+def query_registry_value(root, key_name, value):
+    try:
+        import winreg
+    except ImportError:
+        import _winreg as winreg
+    try:
+        with winreg.OpenKeyEx(root, key_name, 0, winreg.KEY_READ) as key:
+            return get_value_from_tuple(*winreg.QueryValueEx(key, value))
+    except OSError:
+        return None
+
+
+def _find_icacls_exe():
+    if os.name == "nt":
+        paths = [
+            os.path.expandvars(r"%windir%\{0}").format(subdir)
+            for subdir in ("system32", "SysWOW64")
+        ]
+        for path in paths:
+            icacls_path = next(
+                iter(fn for fn in os.listdir(path) if fn.lower() == "icacls.exe"), None
+            )
+            if icacls_path is not None:
+                icacls_path = os.path.join(path, icacls_path)
+                return icacls_path
+    return None
+
+
+def _walk_for_powershell(directory):
+    for _, dirs, files in os.walk(directory):
+        powershell = next(
+            iter(fn for fn in files if fn.lower() == "powershell.exe"), None
+        )
+        if powershell is not None:
+            return os.path.join(directory, powershell)
+        for subdir in dirs:
+            powershell = _walk_for_powershell(os.path.join(directory, subdir))
+            if powershell:
+                return powershell
+    return None
+
+
+def _get_powershell_path():
+    paths = [
+        os.path.expandvars(r"%windir%\{0}\WindowsPowerShell").format(subdir)
+        for subdir in ("SysWOW64", "system32")
+    ]
+    powershell_path = next(iter(_walk_for_powershell(pth) for pth in paths), None)
+    if not powershell_path:
+        powershell_path = sp.run(["where", "powershell"])
+    if powershell_path.stdout:
+        return powershell_path.stdout.strip()
+
+
+def _get_sid_with_powershell():
+    powershell_path = _get_powershell_path()
+    if not powershell_path:
+        return None
+    args = [
+        powershell_path,
+        "-ExecutionPolicy",
+        "Bypass",
+        "-Command",
+        "Invoke-Expression '[System.Security.Principal.WindowsIdentity]::GetCurrent().user | Write-Host'",
+    ]
+    sid = sp.run(args, capture_output=True)
+    return sid.stdout.strip()
+
+
+def _get_sid_from_registry():
+    try:
+        import winreg
+    except ImportError:
+        import _winreg as winreg
+    var_names = ("%USERPROFILE%", "%HOME%")
+    current_user_home = next(iter(os.path.expandvars(v) for v in var_names if v), None)
+    root, subkey = (
+        winreg.HKEY_LOCAL_MACHINE,
+        r"Software\Microsoft\Windows NT\CurrentVersion\ProfileList",
+    )
+    subkey_names = []
+    value = None
+    matching_key = None
+    try:
+        with winreg.OpenKeyEx(root, subkey, 0, winreg.KEY_READ) as key:
+            for i in count():
+                key_name = winreg.EnumKey(key, i)
+                subkey_names.append(key_name)
+                value = query_registry_value(
+                    root, r"{0}\{1}".format(subkey, key_name), "ProfileImagePath"
+                )
+                if value and value.lower() == current_user_home.lower():
+                    matching_key = key_name
+                    break
+    except OSError:
+        pass
+    if matching_key is not None:
+        return matching_key
+
+
+def _get_current_user():
+    fns = (_get_sid_from_registry, _get_sid_with_powershell)
+    for fn in fns:
+        result = fn()
+        if result:
+            return result
+    return None
+
+
+def _wait_for_files(path):  # pragma: no cover
+    """Retry with backoff up to 1 second to delete files from a directory.
+
+    :param str path: The path to crawl to delete files from
+    :return: A list of remaining paths or None
+    :rtype: Optional[List[str]]
+    """
+    timeout = 0.001
+    remaining = []
+    while timeout < 1.0:
+        remaining = []
+        if os.path.isdir(path):
+            L = os.listdir(path)
+            for target in L:
+                _remaining = _wait_for_files(target)
+                if _remaining:
+                    remaining.extend(_remaining)
+            continue
+        try:
+            os.unlink(path)
+        except FileNotFoundError as e:
+            if e.errno == errno.ENOENT:
+                return
+        except (OSError, IOError, PermissionError):  # noqa:B014
+            time.sleep(timeout)
+            timeout *= 2
+            remaining.append(path)
+        else:
+            return
+    return remaining
+
+
+def set_write_bit(fn: str) -> None:
+    """Set read-write permissions for the current user on the target path. Fail
+    silently if the path doesn't exist.
+
+    :param str fn: The target filename or path
+    :return: None
+    """
+    if not os.path.exists(fn):
+        return
+    file_stat = os.stat(fn).st_mode
+    os.chmod(fn, file_stat | stat.S_IRWXU | stat.S_IRWXG | stat.S_IRWXO)
+    if os.name == "nt":
+        user_sid = _get_current_user()
+        icacls_exe = _find_icacls_exe() or "icacls"
+
+        if user_sid:
+            c = sp.run(
+                [
+                    icacls_exe,
+                    "''{}''".format(fn),
+                    "/grant",
+                    "{}:WD".format(user_sid),
+                    "/T",
+                    "/C",
+                    "/Q",
+                ],
+                capture_output=True,
+                # 2020-06-12 Yukihiko Shinoda
+                # There are 3 way to get system default encoding in Stack Overflow.
+                # see: https://stackoverflow.com/questions/37506535/how-to-get-the-system-default-encoding-in-python-2-x
+                # I investigated these way by using Shift-JIS Windows.
+                # >>> import locale
+                # >>> locale.getpreferredencoding()
+                # "cp932" (Shift-JIS)
+                # >>> import sys
+                # >>> sys.getdefaultencoding()
+                # "utf-8"
+                # >>> sys.stdout.encoding
+                # "UTF8"
+                encoding=locale.getpreferredencoding(),
+            )
+            if not c.err and c.returncode == 0:
+                return
+
+    if not os.path.isdir(fn):
+        for path in [fn, os.path.dirname(fn)]:
+            try:
+                os.chflags(path, 0)
+            except AttributeError:
+                pass
+        return None
+    for root, dirs, files in os.walk(fn, topdown=False):
+        for dir_ in [os.path.join(root, d) for d in dirs]:
+            set_write_bit(dir_)
+        for file_ in [os.path.join(root, f) for f in files]:
+            set_write_bit(file_)
+
+
 def make_base_requirements(reqs):
     # type: (Sequence[STRING_TYPE]) -> Set[BaseRequirement]
     requirements = set()
@@ -159,6 +369,96 @@ def make_base_requirements(reqs):
     return requirements
 
 
+def handle_remove_readonly(func, path, exc):
+    """Error handler for shutil.rmtree.
+
+    Windows source repo folders are read-only by default, so this error handler
+    attempts to set them as writeable and then proceed with deletion.
+
+    :param function func: The caller function
+    :param str path: The target path for removal
+    :param Exception exc: The raised exception
+
+    This function will call check :func:`is_readonly_path` before attempting to call
+    :func:`set_write_bit` on the target path and try again.
+    """
+
+    PERM_ERRORS = (errno.EACCES, errno.EPERM, errno.ENOENT)
+    default_warning_message = "Unable to remove file due to permissions restriction: {!r}"
+    # split the initial exception out into its type, exception, and traceback
+    exc_type, exc_exception, exc_tb = exc
+    if is_readonly_path(path):
+        # Apply write permission and call original function
+        set_write_bit(path)
+        try:
+            func(path)
+        except (  # noqa:B014
+            OSError,
+            IOError,
+            FileNotFoundError,
+            PermissionError,
+        ) as e:  # pragma: no cover
+            if e.errno in PERM_ERRORS:
+                if e.errno == errno.ENOENT:
+                    return
+                remaining = None
+                if os.path.isdir(path):
+                    remaining = _wait_for_files(path)
+                if remaining:
+                    warnings.warn(
+                        default_warning_message.format(path),
+                        ResourceWarning,
+                        stacklevel=2,
+                    )
+                else:
+                    func(path, ignore_errors=True)
+                return
+
+    if exc_exception.errno in PERM_ERRORS:
+        set_write_bit(path)
+        remaining = _wait_for_files(path)
+        try:
+            func(path)
+        except (OSError, IOError, FileNotFoundError, PermissionError) as e:  # noqa:B014
+            if e.errno in PERM_ERRORS:
+                if e.errno != errno.ENOENT:  # File still exists
+                    warnings.warn(
+                        default_warning_message.format(path),
+                        ResourceWarning,
+                        stacklevel=2,
+                    )
+            return
+    else:
+        raise exc_exception
+
+
+def rmtree(
+    directory: str, ignore_errors: bool = False, onerror: Optional[Callable] = None
+) -> None:
+    """Stand-in for :func:`~shutil.rmtree` with additional error-handling.
+
+    This version of `rmtree` handles read-only paths, especially in the case of index
+    files written by certain source control systems.
+
+    :param str directory: The target directory to remove
+    :param bool ignore_errors: Whether to ignore errors, defaults to False
+    :param func onerror: An error handling function, defaults to :func:`handle_remove_readonly`
+
+    .. note::
+
+       Setting `ignore_errors=True` may cause this to silently fail to delete the path
+    """
+
+    if onerror is None:
+        onerror = handle_remove_readonly
+    try:
+        shutil.rmtree(directory, ignore_errors=ignore_errors, onerror=onerror)
+    except (IOError, OSError, FileNotFoundError, PermissionError) as exc:  # noqa:B014
+        # Ignore removal failures where the file doesn't exist
+        if exc.errno != errno.ENOENT:
+            raise
+
+
 def suppress_unparsable(func, *args, **kwargs):
     try:
         return func(*args, **kwargs)
@@ -176,7 +476,7 @@ class SetupReader:
     @classmethod
     def read_setup_py(cls, file: Path, raising: bool = True) -> "Dict[str, Any]":
 
-        with file.open(encoding="utf-8") as f:
+        with file.open(encoding="utf-8-sig") as f:
             content = f.read()
 
         body = ast.parse(content).body
@@ -896,13 +1196,12 @@ def run_setup(script_path, egg_base=None):
         # We couldn't import everything needed to run setup
         except Exception:
             python = os.environ.get("PIP_PYTHON_PATH", sys.executable)
-            out, _ = run(
+
+            sp.run(
                 [python, "setup.py"] + args,
                 cwd=target_cwd,
-                block=True,
-                combine_stderr=False,
-                return_object=False,
-                nospin=True,
+                stdout=sp.PIPE,
+                stderr=sp.PIPE,
             )
         finally:
             _setup_stop_after = None
@@ -1534,16 +1833,22 @@ build-backend = "{1}"
             build_location_func(**build_kwargs)
             ireq.ensure_has_source_dir(kwargs["src_dir"])
             location = None
-            if getattr(ireq, "source_dir", None):
+            if ireq.source_dir:
                 location = ireq.source_dir
-            old_unpack_url(
-                link=ireq.link,
-                location=location,
-                download=Downloader(session, "off"),
-                verbosity=1,
-                download_dir=download_dir,
-                hashes=ireq.hashes(True),
-            )
+
+            if ireq.link.is_existing_dir():
+                if os.path.isdir(location):
+                    rmtree(location)
+                _copy_source_tree(ireq.link.file_path, location)
+            else:
+                unpack_url(
+                    link=ireq.link,
+                    location=location,
+                    download=Downloader(session, "off"),
+                    verbosity=1,
+                    download_dir=download_dir,
+                    hashes=ireq.hashes(True),
+                )
         created = cls.create(
             ireq.source_dir, subdirectory=subdir, ireq=ireq, kwargs=kwargs, stack=stack
         )
